@@ -59,8 +59,7 @@ CREATE TABLE user (
   max int(11) NOT NULL default '0',
   warn int(11) NOT NULL default '7',
   inact int(11) NOT NULL default '-1',
-  expire int(11) NOT NULL default '-1',
-  PRIMARY KEY  (user_id)
+  expire int(11) NOT NULL default '-1'
  );
 
 #
