@@ -17,9 +17,9 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-    $Id: group.c,v 1.125 2002/09/09 19:32:24 gmorin Exp $
+    $Id: group.c,v 1.131 2003/05/12 14:59:18 gmorin Exp $
     $Source: /cvsroot/nss-mysql/nss-mysql/src/group.c,v $
-    $Date: 2002/09/09 19:32:24 $
+    $Date: 2003/05/12 14:59:18 $
     $Author: gmorin $
 */
 
@@ -29,16 +29,8 @@
 #include <errno.h>
 #include <grp.h>
 
-#ifdef HAVE_CONFIG_H
-        #include "config.h"
-#endif
-
 #include "nss-mysql.h"
 
-#ifdef INCLUDE_MY_SYS
-        #include <my_pthread.h>
-#endif
-
 #include "lib.h"
 #include "parser.h"
 #include "passwd.h"
@@ -63,6 +55,7 @@ NSS_STATUS _nss_mysql_getgrgid_r (const gid_t gid, struct group *gr,
 static void prepare_forkhandler(void);
 static void parent_forkhandler(void);
 static void child_forkhandler(void);
+static void atexit_handler(void);
 static NSS_STATUS handle_query(struct query * q,struct mysql_auth * mysql_auth,
                 struct group * gr, char * buffer, size_t buflen, int * errnop);
 static char * get_sql_query(struct query * q, struct parse_result * pr,
@@ -116,7 +109,8 @@ char * get_sql_query(struct query * q, struct parse_result * pr,
                 /* we look by name */
                 sql = _nss_mysql_sqlprintf("select %s,%s,%s,%s from %s "
                                 "LEFT JOIN %s on %s=%s LEFT JOIN %s on %s=%s "
-                                "and %s where %s = '%s' and %s",
+                                "and %s where %s = '%s' and %s order by "
+                                "%s,%s desc",
                         goptions->groupnamecolumn,
                         goptions->gidcolumn,
                         goptions->passwordcolumn,
@@ -138,14 +132,18 @@ char * get_sql_query(struct query * q, struct parse_result * pr,
                         /* where */
                         goptions->groupnamecolumn,
                         e_name,
-                        goptions->where[0] ? goptions->where : "1=1");
+                        goptions->where[0] ? goptions->where : "1=1",
+                        /* order by */
+                        goptions->groupnamecolumn,
+                        options->usercolumn);
                 if (call_free)
                         free(e_name);
         } else {
                 /* we look by GID */
                 sql = _nss_mysql_sqlprintf("select %s,%s,%s,%s from %s "
                                 "LEFT JOIN %s on %s=%s LEFT JOIN %s on %s=%s "
-                                "and %s where %s = %d and %s",
+                                "and %s where %s = %d and %s order by %s"
+                                ",%s desc",
                         goptions->groupnamecolumn,
                         goptions->gidcolumn,
                         goptions->passwordcolumn,
@@ -168,7 +166,10 @@ char * get_sql_query(struct query * q, struct parse_result * pr,
                         /* for bulk retrieval, we retrieve every entry */
                         q->bulk ? "1" : goptions->gidcolumn,
                         q->bulk ? 1 : q->gid,
-                        goptions->where[0] ? goptions->where : "1=1");
+                        goptions->where[0] ? goptions->where : "1=1",
+                        /* order by */
+                        goptions->groupnamecolumn,
+                        options->usercolumn);
         }
 
         if (DEBUG)
@@ -220,7 +221,8 @@ NSS_STATUS handle_query(struct query * q,struct mysql_auth * mysql_auth,
         _nss_mysql_set_fork_handler(&forkhandler_isset,&forkhandler_mutex,
                         prepare_forkhandler,
                         parent_forkhandler,
-                        child_forkhandler);
+                        child_forkhandler,
+                        atexit_handler);
 
         if (! _nss_mysql_check_connection(mysql_auth,&options.con)) {
                 status = NSS_STATUS_UNAVAIL;
@@ -298,8 +300,8 @@ NSS_STATUS _nss_mysql_group_result_to_struct(struct group * gr,
         
         if (max && num_rows > max) {
                 _nss_mysql_log(LOG_ERR,"_nss_mysql_group_result_to_struct: "
-                                "Warning query returned %d rows, was "
-                                "expecting only %d. Results will be "
+                                "Warning query returned %lu rows, was "
+                                "expecting only %lu. Results will be "
                                 "incorrect.",num_rows,max);
         }
         
@@ -382,12 +384,15 @@ NSS_STATUS _nss_mysql_group_result_to_struct(struct group * gr,
                                                         gr->gr_name);
                                 break;
                         } else {
-                                /* empty user name in the DB */
+                                /* empty user name in the DB 
                                 _nss_mysql_log(LOG_ERR,"Empty or NULL member "
                                         "for group %s(%d). Fix your "
                                         "database", gr->gr_name,
                                         gr->gr_gid);
                                 return NSS_STATUS_UNAVAIL;
+                                */
+                                /* should be a disabled user */
+                                goto end_of_loop;
                         }
                 }
 
@@ -408,11 +413,11 @@ NSS_STATUS _nss_mysql_group_result_to_struct(struct group * gr,
                 /* we save the string address in the array */
                 *addrptr = p;
                 ++addrptr;
-                last = mysql_row_tell(result);
-
                 if (DEBUG) _nss_mysql_log(LOG_ERR,"%s is a member of %s"
                                 "(%d)",sql_row[3],gr->gr_name,
                                 gr->gr_gid);
+end_of_loop:
+                last = mysql_row_tell(result);
         }
         if ((void *) addrptr >= (void *) end_of_buf)
                 goto out_nomem;
@@ -423,6 +428,7 @@ NSS_STATUS _nss_mysql_group_result_to_struct(struct group * gr,
                 _nss_mysql_log(LOG_ERR,"_nss_mysql_group_result_to_struct: "
                                 "success, exiting");
 
+        *errnop = 0;
         return NSS_STATUS_SUCCESS;
 
 out_nomem:
@@ -496,7 +502,8 @@ NSS_STATUS _nss_mysql_initgroups (const char *user, gid_t group,
         _nss_mysql_set_fork_handler(&forkhandler_isset,&forkhandler_mutex,
                         prepare_forkhandler,
                         parent_forkhandler,
-                        child_forkhandler);
+                        child_forkhandler,
+                        atexit_handler);
         
         if (! _nss_mysql_check_connection(&initgroups_auth,&options.con)) {
                 _nss_mysql_free_users(&options);
@@ -801,6 +808,20 @@ void child_forkhandler(void) {
                 _nss_mysql_log(LOG_ERR,"group.c: end of child_forkhandler");
 }
 
+
+void atexit_handler(void) {
+        if (DEBUG)
+                _nss_mysql_log(LOG_ERR,"group.c: calling atexit_handler");
+        pthread_mutex_lock(nam_auth.mutex);
+        _nss_mysql_db_close(&nam_auth.mysql);
+        pthread_mutex_lock(gid_auth.mutex);
+        _nss_mysql_db_close(&gid_auth.mysql);
+        pthread_mutex_lock(initgroups_auth.mutex);
+        _nss_mysql_db_close(&initgroups_auth.mysql);
+        if (DEBUG)
+                _nss_mysql_log(LOG_ERR,"group.c: end of atexit_handler");
+}
+
 #ifdef TEST
 
 int main(void) {
