@@ -1,6 +1,7 @@
 /*
     Copyright (C) 2000 Steve Brown
-    Copyright (C) 2000,2001,2002 Guillaume Morin, Alcve
+    Copyright (C) 2000,2001,2002 Alcve
+    Copyright (C) 2000,2001,2002,2005 Guillaume Morin
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,9 +18,9 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-    $Id: lib.c,v 1.62 2002/09/06 09:50:09 gmorin Exp $
+    $Id: lib.c,v 1.72 2005/03/19 02:50:39 gmorin Exp $
     $Source: /cvsroot/nss-mysql/nss-mysql/src/lib.c,v $
-    $Date: 2002/09/06 09:50:09 $
+    $Date: 2005/03/19 02:50:39 $
     $Author: gmorin $
 */
 
@@ -35,10 +36,6 @@
 #include <ctype.h>
 #include <unistd.h>
 
-#ifdef HAVE_CONFIG_H
-        #include "config.h"
-#endif
-
 #include "lib.h"
 
 static char * parse_host (char * host, char ** unix_socket, int * port);
@@ -46,7 +43,7 @@ static void create_pthread_key(void);
 
 /*EOP*/
 
-static pthread_once_t key_once = PTHREAD_ONCE_INIT;
+static pthread_once_t key_once = NSS_MYSQL_PTHREAD_ONCE_INIT;
 static pthread_key_t tsd_key;
 
 /* Creates the key for the thread specific data */
@@ -118,8 +115,8 @@ NSS_STATUS _nss_mysql_send_query(struct mysql_auth * auth,char * query,
 
         if (mysql_query(auth->mysql, query)) {
                 _nss_mysql_log(LOG_ERR,"_nss_mysql_send_query: "
-                                "mysql_query failed: %s",
-                                mysql_error(auth->mysql));
+                                "mysql_query failed: %s for %s",
+                                mysql_error(auth->mysql),query);
                 if (auth->mutex)
                         pthread_mutex_unlock(auth->mutex);
                 goto out_unavail;
@@ -158,7 +155,8 @@ out_unavail:
 void _nss_mysql_set_fork_handler(int * isset,pthread_mutex_t * mutex,
                 void (*prepare)(void),
                 void (*parent)(void),
-                void (*child)(void)) {
+                void (*child)(void),
+                void (*atexit_handler)(void)) {
         if (DEBUG)
                 _nss_mysql_log(LOG_ERR,"_nss_mysql_set_fork_handler: called");
       
@@ -176,6 +174,14 @@ void _nss_mysql_set_fork_handler(int * isset,pthread_mutex_t * mutex,
                                 _nss_mysql_log(LOG_ERR,"_nss_mysql_set_fork_"
                                                 "handler: pthread_atfork "
                                                 "failed: %s",strerror(errno));
+                        if (DEBUG && atexit_handler)
+                                _nss_mysql_log(LOG_ERR,"_nss_mysql_set_fork_"
+                                                "handler: setting the atexit "
+                                                "handler");
+                        if (atexit_handler && atexit(atexit_handler) != 0)
+                                _nss_mysql_log(LOG_ERR,"_nss_mysql_set_fork_"
+                                                "handler: atexit "
+                                                "failed: %s",strerror(errno));
                         *isset = 1;
                 }
                 pthread_mutex_unlock(mutex);
@@ -319,6 +325,9 @@ void _nss_mysql_log(int err, const char *format, ...) {
 */
 
 void _nss_mysql_db_close (MYSQL ** mysql_auth) {
+        if (DEBUG)
+                _nss_mysql_log(LOG_ERR,"_nss_mysql_db_close: called for %p",
+                                *mysql_auth);
         if (*mysql_auth == NULL) {
                 if (DEBUG) _nss_mysql_log(LOG_ERR,"_nss_mysql_db_close: called "
                                 "with a NULL pointer");
@@ -327,6 +336,8 @@ void _nss_mysql_db_close (MYSQL ** mysql_auth) {
         mysql_close(*mysql_auth);
         /* Trust no one */
         *mysql_auth = NULL;
+        if (DEBUG)
+                _nss_mysql_log(LOG_ERR,"_nss_mysql_db_close: exiting");
 }
 
 /* MySQL access functions */
@@ -535,8 +546,8 @@ int _nss_mysql_check_connection(struct mysql_auth * m,
                 int ping = 0;
                 if (DEBUG)
                         _nss_mysql_log(LOG_ERR,"_nss_mysql_check_connection: "
-                                        "found a saved MYSQL * (%p)",
-                                        m->mysql);
+                                        "found a saved MYSQL * (%p) by %d",
+                                        m->mysql,m->pid);
 
                 if (m->pid == 0) {
                         force_reconnect = 1;
@@ -546,7 +557,7 @@ int _nss_mysql_check_connection(struct mysql_auth * m,
                                                 "called (threaded app). "
                                                 "Reconnecting");
                 } else if (m->pid != getpid()) {
-                        if (pthread_getspecific(tsd_key) == (void *) 1) {
+                        if (pthread_getspecific(tsd_key) == (void *) 0xdeadbeef) {
                                 force_reconnect = 1;
                                 if (DEBUG)
                                         _nss_mysql_log(LOG_ERR,"_nss_mysql_"
@@ -586,7 +597,6 @@ int _nss_mysql_check_connection(struct mysql_auth * m,
 
         /* DB connection */
         if (m->mysql == NULL || force_reconnect) {
-                pthread_setspecific(tsd_key,(void *) 1);
                 if (DEBUG)
                         _nss_mysql_log(LOG_ERR,"_nss_mysql_check_connection: "
                                         "Initiating a new connection");
@@ -594,6 +604,7 @@ int _nss_mysql_check_connection(struct mysql_auth * m,
                         pthread_mutex_unlock(m->mutex);
                         return 0;
                 }
+                pthread_setspecific(tsd_key,(void *) 0xdeadbeef);
         }
 
         if (DEBUG)
