@@ -1,7 +1,7 @@
 /*
-    $Id: lib.h,v 1.27 2002/09/04 23:22:48 gmorin Exp $
+    $Id: lib.h,v 1.28 2002/10/07 19:18:06 gmorin Exp $
     $Source: /cvsroot/nss-mysql/nss-mysql/src/lib.h,v $
-    $Date: 2002/09/04 23:22:48 $
+    $Date: 2002/10/07 19:18:06 $
     $Author: gmorin $
 */
 
@@ -28,7 +28,8 @@ extern int _nss_mysql_check_connection(struct mysql_auth *,
 void _nss_mysql_set_fork_handler(int * isset,pthread_mutex_t * mutex,
                 void (*prepare)(void),
                 void (*parent)(void),
-                void (*child)(void));
+                void (*child)(void),
+                void (*atexit)(void));
 
 NSS_STATUS _nss_mysql_send_query(struct mysql_auth * auth,char * query,
                 MYSQL_RES ** result, int * errnop);
