@@ -1,7 +1,10 @@
 #ifndef _HAVE_NSS_MYSQL_H
 #define _HAVE_NSS_MYSQL_H
 
-static char nss_mysql_version[] = "NSS-MySQL version 0.43";
+#ifdef HAVE_CONFIG_H
+        #include <config.h>
+        static const char nss_mysql_version[] = PACKAGE_STRING;
+#endif
 
 #include <mysql.h>
 #include <sys/types.h>
@@ -54,4 +57,14 @@ typedef enum nss_status NSS_STATUS;
         #define NSS_MYSQL_PTHREAD_MUTEX_INITIALIZER PTHREAD_MUTEX_INITIALIZER
 #endif
 
+#ifdef PTHREAD_ONCE_INIT_NEEDS_BRACKETS
+        #define NSS_MYSQL_PTHREAD_ONCE_INIT {PTHREAD_ONCE_INIT}
+#else
+        #define NSS_MYSQL_PTHREAD_ONCE_INIT PTHREAD_ONCE_INIT
+#endif
+
+#ifdef INCLUDE_MY_SYS
+        #include <my_pthread.h>
+#endif
+
 #endif /* _HAVE_NSS_MYSQL_H */
