@@ -1,6 +1,7 @@
 /*
     Copyright (C) 2000 Steve Brown
     Copyright (C) 2000,2001,2002 Guillaume Morin, Alcve
+    Copyright (C) 2003 Guillaume Morin
         
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,9 +18,9 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-    $Id: parser.c,v 1.69 2002/08/30 14:53:38 gmorin Exp $
+    $Id: parser.c,v 1.73 2003/01/31 13:19:28 gmorin Exp $
     $Source: /cvsroot/nss-mysql/nss-mysql/src/parser.c,v $
-    $Date: 2002/08/30 14:53:38 $
+    $Date: 2003/01/31 13:19:28 $
     $Author: gmorin $
 */
 
@@ -31,10 +32,6 @@
 #include <sys/types.h>
 #include <errno.h>
 
-#ifdef HAVE_CONFIG_H
-        #include "config.h"
-#endif
-
 #include "nss-mysql.h"
 #include "parser.h"
 #include "nss-shadow.h"
@@ -241,9 +238,11 @@ int _nss_mysql_read_conf_file(char * section,struct parse_result *pr) {
         }
         
         if (fd == NULL) {
-                _nss_mysql_log(LOG_ERR, "Cannot open %s configuration file: "
-                                "%s. (euid=%d, uid=%d)", conf_file,
-                                strerror(errno),geteuid,getuid);
+                if (DEBUG)
+                        _nss_mysql_log(LOG_ERR, "Cannot open %s configuration"
+                                        "file: %s. (euid=%d, uid=%d)", 
+                                        conf_file,strerror(errno),geteuid(),
+                                        getuid());
                 return 0;
         }
 
