@@ -17,9 +17,9 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-    $Id: passwd.c,v 1.102 2002/09/09 19:32:24 gmorin Exp $
+    $Id: passwd.c,v 1.106 2003/05/12 14:59:18 gmorin Exp $
     $Source: /cvsroot/nss-mysql/nss-mysql/src/passwd.c,v $
-    $Date: 2002/09/09 19:32:24 $
+    $Date: 2003/05/12 14:59:18 $
     $Author: gmorin $
 */
 
@@ -29,16 +29,8 @@
 #include <string.h>
 #include <errno.h>
 
-#ifdef HAVE_CONFIG_H
-        #include "config.h"
-#endif
-
 #include "nss-mysql.h"
 
-#ifdef INCLUDE_MY_SYS
-        #include <my_pthread.h>
-#endif
-
 #include "lib.h"
 #include "parser.h"
 #include "passwd.h"
@@ -60,6 +52,7 @@ NSS_STATUS _nss_mysql_getpwnam_r(const char *,struct passwd *,char *,size_t,int
 static void prepare_forkhandler(void);
 static void parent_forkhandler(void);
 static void child_forkhandler(void);
+static void atexit_handler(void);
 static NSS_STATUS handle_query(struct query * q,struct mysql_auth * mysql_auth,
                 struct passwd * pw, char * buffer, size_t buflen,int * errnop);
 static char * get_sql_query(struct query * q,struct parse_result * pr,
@@ -215,7 +208,8 @@ NSS_STATUS handle_query(struct query * q,struct mysql_auth * mysql_auth,
         _nss_mysql_set_fork_handler(&forkhandler_isset,&forkhandler_mutex,
                         prepare_forkhandler,
                         parent_forkhandler,
-                        child_forkhandler);
+                        child_forkhandler,
+                        atexit_handler);
 
         if (! _nss_mysql_check_connection(mysql_auth,&options.con)) {
                 status = NSS_STATUS_UNAVAIL;
@@ -286,8 +280,9 @@ NSS_STATUS _nss_mysql_passwd_result_to_struct(struct passwd *pw,
         }
 
         if (max && rows > max) {
-                _nss_mysql_log(LOG_ERR,"Warning: lookup returned %d rows, I "
-                                "was expecting just %d.",i,max);
+                _nss_mysql_log(LOG_ERR,"_nss_mysql_passwd_result_to_struct:"
+                                "Warning: lookup returned %lu rows, I "
+                                "was expecting just %lu.",rows,max);
         }
 
         sql_row = mysql_fetch_row(result);
@@ -524,5 +519,14 @@ void child_forkhandler(void) {
                 _nss_mysql_log(LOG_ERR,"passwd.c: end of child_forkhandler");
 }
 
-
+void atexit_handler(void) {
+        if (DEBUG)
+                _nss_mysql_log(LOG_ERR,"passwd.c: calling atexit_handler");
+        pthread_mutex_lock(nam_auth.mutex);
+        _nss_mysql_db_close(&nam_auth.mysql);
+        pthread_mutex_lock(uid_auth.mutex);
+        _nss_mysql_db_close(&uid_auth.mysql);
+        if (DEBUG)
+                _nss_mysql_log(LOG_ERR,"passwd.c: end of atexit_handler");
+}
 /* EOF */
